#!/usr/bin/python2 -tt
#
# Copyright (c) 2007 Twin Peaks Software, Inc.
#
#ident  "@(#)   post_install.py     1.2     08/04/07    TwinPeaksSoft"

import os
import os.path

import fcntl
import socket
import struct
import array

import urllib
import httplib

WSERVER = "www.TwinPeakSoft.com"
#WSERVER = "toad"
HTTP_ROOT = "http://" + WSERVER
LIC_LOC = "/py_files/userinfo.py/sendlicense"
INFO_LOC = "/py_files/userinfo.py/getinfo"
LIC_HTTP = HTTP_ROOT + LIC_LOC

LIC_FILE = "/var/mfs/TPSmfs/license_dir/tpsmfs.lic"

ERROR_MSG = 'The installtion program can not install the license correctly, '\
	'\nplease email support@TwinPeakSoft.com to get a trial license'

STATUS_SUCCEED = "succeed"              # Succeed to install license file
STATUS_EXIST = "already_exist"          # License file already exist
STATUS_FAILED = "install_failed"        # Failed to install license file


# Create a directory if not exist
def create_dir(dir_name):
    if os.path.exists(dir_name):
        return

    parent = os.path.dirname(dir_name)
    if not os.path.exists(parent):
        create_dir(parent)

    os.mkdir(dir_name)


# Get all the 'up' network interface
def get_all_interfaces():
    max_possible = 128  # arbitrary. raise if needed.
    bytes = max_possible * 32
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    names = array.array('B', '\0' * bytes)
    outbytes = struct.unpack('iL', fcntl.ioctl(
        s.fileno(),
        0x8912,  # SIOCGIFCONF
        struct.pack('iL', bytes, names.buffer_info()[0])
    ))[0]
    namestr = names.tostring()
    return [namestr[i:i+32].split('\0', 1)[0] for i in range(0, outbytes, 32)]


# Get ip address of a interface
def get_ip_address(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])

    
# Get hardware address of a interface
def get_mac_address(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', ifname[:15]))
    mac_addr = []
    for char in info[18:24]:
        hdigit = hex(ord(char))[2:]
        mac_addr.append(str(hdigit))
    
    return ":".join(mac_addr)


# Get host name, ip address and MAC address of a machine
def get_info():
    # Get host name
    hostname = socket.getfqdn(socket.gethostname())

    # Get ip address and mac address
    ipaddr = []
    hwaddr = []
    if_names = get_all_interfaces()
    for ifname in if_names:
        if ifname.find("eth") != 0:
            continue
        hwaddr.append(get_mac_address(ifname))
        ipaddr.append(get_ip_address(ifname))

    return (hostname, ipaddr, hwaddr)


# Get license file
def get_lic():
    conn = httplib.HTTPConnection(WSERVER)
    conn.request("GET", LIC_LOC)
    response = conn.getresponse()
    return response.read()


# Send back machine information
def send_info(hostname, status, ipaddr, hwaddr):
    params = urllib.urlencode({'host': hostname, 'status': status,
            'ipaddr': ipaddr, 'hwaddr': hwaddr})
    headers = {"Content-type": "application/x-www-form-urlencoded",
                "Accept": "text/plain"}
    conn = httplib.HTTPConnection(WSERVER)
    conn.request("POST", INFO_LOC, params, headers)
    response = conn.getresponse()
    if response.status != 200:
        return 0
    data = response.read()
    conn.close()
    return 1


def main():
    # Install license file
    if os.path.exists(LIC_FILE):
        status = STATUS_EXIST
    else:
        try:
            # Get the license file
            lic = get_lic()
            # print lic

            # Create the license file
            dir = os.path.dirname(LIC_FILE)
            create_dir(dir)
            lic_file = open(LIC_FILE, 'w')

            # Copy the file
            lic_file.write(lic)

            status = STATUS_SUCCEED
        except:
            # raise
            status = STATUS_FAILED

    # Gether user information
    (hostname, ipaddr, hwaddr) = get_info()

    # Send back user information
    try:
        if send_info(hostname, status, ','.join(ipaddr), ','.join(hwaddr)) == 0:
            print ERROR_MSG
            return 1
    except:
        # raise
        print ERROR_MSG
        return 1

    if status == STATUS_SUCCEED:
        return 0
    if status == STATUS_EXIST:
        return 0
    else:
        print ERROR_MSG
        return 1

# main entrence
if __name__ == '__main__':
    main()
